@@ -118,23 +118,23 @@ class BinaryReaderIR : public BinaryReaderNop {
   Result OnLocalDecl(Index decl_index, Index count, Type type) override;
 
   Result OnAtomicLoadExpr(Opcode opcode,
-                          uint32_t alignment_log2,
+                          Address alignment_log2,
                           Address offset) override;
   Result OnAtomicStoreExpr(Opcode opcode,
-                           uint32_t alignment_log2,
+                           Address alignment_log2,
                            Address offset) override;
   Result OnAtomicRmwExpr(Opcode opcode,
-                         uint32_t alignment_log2,
+                         Address alignment_log2,
                          Address offset) override;
   Result OnAtomicRmwCmpxchgExpr(Opcode opcode,
-                                uint32_t alignment_log2,
+                                Address alignment_log2,
                                 Address offset) override;
   Result OnAtomicWaitExpr(Opcode opcode,
-                          uint32_t alignment_log2,
+                          Address alignment_log2,
                           Address offset) override;
   Result OnAtomicFenceExpr(uint32_t consistency_model) override;
   Result OnAtomicNotifyExpr(Opcode opcode,
-                            uint32_t alignment_log2,
+                            Address alignment_log2,
                             Address offset) override;
   Result OnBinaryExpr(Opcode opcode) override;
   Result OnBlockExpr(Type sig_type) override;
@@ -163,7 +163,7 @@ class BinaryReaderIR : public BinaryReaderNop {
   Result OnI64ConstExpr(uint64_t value) override;
   Result OnIfExpr(Type sig_type) override;
   Result OnLoadExpr(Opcode opcode,
-                    uint32_t alignment_log2,
+                    Address alignment_log2,
                     Address offset) override;
   Result OnLocalGetExpr(Index local_index) override;
   Result OnLocalSetExpr(Index local_index) override;
@@ -191,7 +191,7 @@ class BinaryReaderIR : public BinaryReaderNop {
   Result OnReturnExpr() override;
   Result OnSelectExpr(Type result_type) override;
   Result OnStoreExpr(Opcode opcode,
-                     uint32_t alignment_log2,
+                     Address alignment_log2,
                      Address offset) override;
   Result OnThrowExpr(Index event_index) override;
   Result OnTryExpr(Type sig_type) override;
@@ -202,7 +202,7 @@ class BinaryReaderIR : public BinaryReaderNop {
   Result OnSimdLaneOpExpr(Opcode opcode, uint64_t value) override;
   Result OnSimdShuffleOpExpr(Opcode opcode, v128 value) override;
   Result OnLoadSplatExpr(Opcode opcode,
-                         uint32_t alignment_log2,
+                         Address alignment_log2,
                          Address offset) override;
 
   Result OnElemSegmentCount(Index count) override;
@@ -260,6 +260,8 @@ class BinaryReaderIR : public BinaryReaderNop {
                           Index section_index) override;
   Result OnEventSymbol(Index index, uint32_t flags, string_view name,
                         Index event_index) override;
+  Result OnTableSymbol(Index index, uint32_t flags, string_view name,
+                       Index table_index) override;
 
  private:
   Location GetLocation() const;
@@ -633,35 +635,35 @@ Result BinaryReaderIR::OnLocalDecl(Index decl_index, Index count, Type type) {
 }
 
 Result BinaryReaderIR::OnAtomicLoadExpr(Opcode opcode,
-                                        uint32_t alignment_log2,
+                                        Address alignment_log2,
                                         Address offset) {
   return AppendExpr(
       MakeUnique<AtomicLoadExpr>(opcode, 1 << alignment_log2, offset));
 }
 
 Result BinaryReaderIR::OnAtomicStoreExpr(Opcode opcode,
-                                         uint32_t alignment_log2,
+                                         Address alignment_log2,
                                          Address offset) {
   return AppendExpr(
       MakeUnique<AtomicStoreExpr>(opcode, 1 << alignment_log2, offset));
 }
 
 Result BinaryReaderIR::OnAtomicRmwExpr(Opcode opcode,
-                                       uint32_t alignment_log2,
+                                       Address alignment_log2,
                                        Address offset) {
   return AppendExpr(
       MakeUnique<AtomicRmwExpr>(opcode, 1 << alignment_log2, offset));
 }
 
 Result BinaryReaderIR::OnAtomicRmwCmpxchgExpr(Opcode opcode,
-                                              uint32_t alignment_log2,
+                                              Address alignment_log2,
                                               Address offset) {
   return AppendExpr(
       MakeUnique<AtomicRmwCmpxchgExpr>(opcode, 1 << alignment_log2, offset));
 }
 
 Result BinaryReaderIR::OnAtomicWaitExpr(Opcode opcode,
-                                        uint32_t alignment_log2,
+                                        Address alignment_log2,
                                         Address offset) {
   return AppendExpr(
       MakeUnique<AtomicWaitExpr>(opcode, 1 << alignment_log2, offset));
@@ -672,7 +674,7 @@ Result BinaryReaderIR::OnAtomicFenceExpr(uint32_t consistency_model) {
 }
 
 Result BinaryReaderIR::OnAtomicNotifyExpr(Opcode opcode,
-                                          uint32_t alignment_log2,
+                                          Address alignment_log2,
                                           Address offset) {
   return AppendExpr(
       MakeUnique<AtomicNotifyExpr>(opcode, 1 << alignment_log2, offset));
@@ -841,7 +843,7 @@ Result BinaryReaderIR::OnIfExpr(Type sig_type) {
 }
 
 Result BinaryReaderIR::OnLoadExpr(Opcode opcode,
-                                  uint32_t alignment_log2,
+                                  Address alignment_log2,
                                   Address offset) {
   return AppendExpr(MakeUnique<LoadExpr>(opcode, 1 << alignment_log2, offset));
 }
@@ -949,7 +951,7 @@ Result BinaryReaderIR::OnLocalSetExpr(Index local_index) {
 }
 
 Result BinaryReaderIR::OnStoreExpr(Opcode opcode,
-                                   uint32_t alignment_log2,
+                                   Address alignment_log2,
                                    Address offset) {
   return AppendExpr(MakeUnique<StoreExpr>(opcode, 1 << alignment_log2, offset));
 }
@@ -1016,7 +1018,7 @@ Result BinaryReaderIR::OnSimdShuffleOpExpr(Opcode opcode, v128 value) {
 }
 
 Result BinaryReaderIR::OnLoadSplatExpr(Opcode opcode,
-                                       uint32_t alignment_log2,
+                                       Address alignment_log2,
                                        Address offset) {
   return AppendExpr(
       MakeUnique<LoadSplatExpr>(opcode, 1 << alignment_log2, offset));
@@ -1277,6 +1279,10 @@ Result BinaryReaderIR::OnDataSymbol(Index index, uint32_t flags,
     // the whole segment.
     return Result::Ok;
   }
+  if (segment >= module_->data_segments.size()) {
+    PrintError("invalid data segment index: %" PRIindex, segment);
+    return Result::Error;
+  }
   DataSegment* seg = module_->data_segments[segment];
   std::string dollar_name =
       GetUniqueName(&module_->data_segment_bindings, MakeDollarName(name));
@@ -1290,6 +1296,10 @@ Result BinaryReaderIR::OnFunctionSymbol(Index index, uint32_t flags,
   if (name.empty()) {
     return Result::Ok;
   }
+  if (func_index >= module_->funcs.size()) {
+    PrintError("invalid function index: %" PRIindex, func_index);
+    return Result::Error;
+  }
   Func* func = module_->funcs[func_index];
   if (!func->name.empty()) {
     // The name section has already named this function.
@@ -1307,6 +1317,10 @@ Result BinaryReaderIR::OnGlobalSymbol(Index index, uint32_t flags,
   if (name.empty()) {
     return Result::Ok;
   }
+  if (global_index >= module_->globals.size()) {
+    PrintError("invalid global index: %" PRIindex, global_index);
+    return Result::Error;
+  }
   Global* glob = module_->globals[global_index];
   std::string dollar_name =
       GetUniqueName(&module_->global_bindings, MakeDollarName(name));
@@ -1325,6 +1339,10 @@ Result BinaryReaderIR::OnEventSymbol(Index index, uint32_t flags,
   if (name.empty()) {
     return Result::Ok;
   }
+  if (event_index >= module_->events.size()) {
+    PrintError("invalid event index: %" PRIindex, event_index);
+    return Result::Error;
+  }
   Event* event = module_->events[event_index];
   std::string dollar_name =
       GetUniqueName(&module_->event_bindings, MakeDollarName(name));
@@ -1333,6 +1351,23 @@ Result BinaryReaderIR::OnEventSymbol(Index index, uint32_t flags,
   return Result::Ok;
 }
 
+Result BinaryReaderIR::OnTableSymbol(Index index, uint32_t flags,
+                                     string_view name, Index table_index) {
+  if (name.empty()) {
+    return Result::Ok;
+  }
+  if (table_index >= module_->tables.size()) {
+    PrintError("invalid table index: %" PRIindex, table_index);
+    return Result::Error;
+  }
+  Table* table = module_->tables[table_index];
+  std::string dollar_name =
+      GetUniqueName(&module_->table_bindings, MakeDollarName(name));
+  table->name = dollar_name;
+  module_->table_bindings.emplace(dollar_name, Binding(table_index));
+  return Result::Ok;
+}
+
 }  // end anonymous namespace
 
 Result ReadBinaryIr(const char* filename,
