@@ -36,7 +36,9 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   }
 
   /* Custom section */
-  Result BeginCustomSection(Offset size, string_view section_name) override {
+  Result BeginCustomSection(Index section_index,
+                            Offset size,
+                            string_view section_name) override {
     return Result::Ok;
   }
   Result EndCustomSection() override { return Result::Ok; }
@@ -192,32 +194,32 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   Result OnOpcodeBlockSig(Type sig_type) override { return Result::Ok; }
   Result OnOpcodeType(Type type) override { return Result::Ok; }
   Result OnAtomicLoadExpr(Opcode opcode,
-                          uint32_t alignment_log2,
+                          Address alignment_log2,
                           Address offset) override {
     return Result::Ok;
   }
   Result OnAtomicStoreExpr(Opcode opcode,
-                           uint32_t alignment_log2,
+                           Address alignment_log2,
                            Address offset) override {
     return Result::Ok;
   }
   Result OnAtomicRmwExpr(Opcode opcode,
-                         uint32_t alignment_log2,
+                         Address alignment_log2,
                          Address offset) override {
     return Result::Ok;
   }
   Result OnAtomicRmwCmpxchgExpr(Opcode opcode,
-                                uint32_t alignment_log2,
+                                Address alignment_log2,
                                 Address offset) override {
     return Result::Ok;
   }
-  Result OnAtomicWaitExpr(Opcode, uint32_t, Address) override {
+  Result OnAtomicWaitExpr(Opcode, Address, Address) override {
     return Result::Ok;
   }
   Result OnAtomicFenceExpr(uint32_t) override {
     return Result::Ok;
   }
-  Result OnAtomicNotifyExpr(Opcode, uint32_t, Address) override {
+  Result OnAtomicNotifyExpr(Opcode, Address, Address) override {
     return Result::Ok;
   }
   Result OnBinaryExpr(Opcode opcode) override { return Result::Ok; }
@@ -250,7 +252,7 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   Result OnI64ConstExpr(uint64_t value) override { return Result::Ok; }
   Result OnIfExpr(Type sig_type) override { return Result::Ok; }
   Result OnLoadExpr(Opcode opcode,
-                    uint32_t alignment_log2,
+                    Address alignment_log2,
                     Address offset) override {
     return Result::Ok;
   }
@@ -286,7 +288,7 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   Result OnReturnExpr() override { return Result::Ok; }
   Result OnSelectExpr(Type result_type) override { return Result::Ok; }
   Result OnStoreExpr(Opcode opcode,
-                     uint32_t alignment_log2,
+                     Address alignment_log2,
                      Address offset) override {
     return Result::Ok;
   }
@@ -304,7 +306,7 @@ class BinaryReaderNop : public BinaryReaderDelegate {
     return Result::Ok;
   }
   Result OnLoadSplatExpr(Opcode opcode,
-                         uint32_t alignment_log2,
+                         Address alignment_log2,
                          Address offset) override {
     return Result::Ok;
   }
@@ -398,6 +400,18 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   }
   Result EndNamesSection() override { return Result::Ok; }
 
+  Result OnNameSubsection(Index index,
+                          NameSectionSubsection subsection_type,
+                          Offset subsection_size) override {
+    return Result::Ok;
+  }
+  Result OnNameCount(Index num_names) override { return Result::Ok; }
+  Result OnNameEntry(NameSectionSubsection type,
+                     Index index,
+                     string_view name) override {
+    return Result::Ok;
+  }
+
   /* Reloc section */
   Result BeginRelocSection(Offset size) override { return Result::Ok; }
   Result OnRelocCount(Index count, Index section_code) override {
@@ -470,10 +484,16 @@ class BinaryReaderNop : public BinaryReaderDelegate {
                        Index event_index) override {
     return Result::Ok;
   }
+  Result OnTableSymbol(Index index,
+                       uint32_t flags,
+                       string_view name,
+                       Index table_index) override {
+    return Result::Ok;
+  }
   Result OnSegmentInfoCount(Index count) override { return Result::Ok; }
   Result OnSegmentInfo(Index index,
                        string_view name,
-                       uint32_t alignment,
+                       Address alignment,
                        uint32_t flags) override {
     return Result::Ok;
   }
