@@ -43,13 +43,17 @@ const char* g_kind_name[] = {"func", "table", "memory", "global", "event"};
 WABT_STATIC_ASSERT(WABT_ARRAY_SIZE(g_kind_name) == kExternalKindCount);
 
 const char* g_reloc_type_name[] = {
-    "R_WASM_FUNCTION_INDEX_LEB",  "R_WASM_TABLE_INDEX_SLEB",
-    "R_WASM_TABLE_INDEX_I32",     "R_WASM_MEMORY_ADDR_LEB",
-    "R_WASM_MEMORY_ADDR_SLEB",    "R_WASM_MEMORY_ADDR_I32",
-    "R_WASM_TYPE_INDEX_LEB",      "R_WASM_GLOBAL_INDEX_LEB",
-    "R_WASM_FUNCTION_OFFSET_I32", "R_WASM_SECTION_OFFSET_I32",
-    "R_WASM_EVENT_INDEX_LEB",     "R_WASM_MEMORY_ADDR_REL_SLEB",
-    "R_WASM_TABLE_INDEX_REL_SLEB",
+    "R_WASM_FUNCTION_INDEX_LEB",   "R_WASM_TABLE_INDEX_SLEB",
+    "R_WASM_TABLE_INDEX_I32",      "R_WASM_MEMORY_ADDR_LEB",
+    "R_WASM_MEMORY_ADDR_SLEB",     "R_WASM_MEMORY_ADDR_I32",
+    "R_WASM_TYPE_INDEX_LEB",       "R_WASM_GLOBAL_INDEX_LEB",
+    "R_WASM_FUNCTION_OFFSET_I32",  "R_WASM_SECTION_OFFSET_I32",
+    "R_WASM_EVENT_INDEX_LEB",      "R_WASM_MEMORY_ADDR_REL_SLEB",
+    "R_WASM_TABLE_INDEX_REL_SLEB", "R_WASM_GLOBAL_INDEX_I32",
+    "R_WASM_MEMORY_ADDR_LEB64",    "R_WASM_MEMORY_ADDR_SLEB64",
+    "R_WASM_MEMORY_ADDR_I64",      "R_WASM_MEMORY_ADDR_REL_SLEB64",
+    "R_WASM_TABLE_INDEX_SLEB64",   "R_WASM_TABLE_INDEX_I64",
+    "R_WASM_TABLE_NUMBER_LEB",
 };
 WABT_STATIC_ASSERT(WABT_ARRAY_SIZE(g_reloc_type_name) == kRelocTypeCount);
 
