@@ -310,9 +310,9 @@ static ValueType ToWabtValueType(wasm_valkind_t kind) {
     case WASM_F64:
       return ValueType::F64;
     case WASM_ANYREF:
-      return ValueType::Anyref;
+      return ValueType::ExternRef;
     case WASM_FUNCREF:
-      return ValueType::Funcref;
+      return ValueType::FuncRef;
     default:
       TRACE("unexpected wasm_valkind_t: %d", kind);
       WABT_UNREACHABLE;
@@ -330,11 +330,9 @@ static wasm_valkind_t FromWabtValueType(ValueType type) {
       return WASM_F32;
     case ValueType::F64:
       return WASM_F64;
-    case ValueType::Anyref:
-    case ValueType::Hostref:
-    case ValueType::Nullref:
+    case ValueType::ExternRef:
       return WASM_ANYREF;
-    case ValueType::Funcref:
+    case ValueType::FuncRef:
       return WASM_FUNCREF;
     default:
       WABT_UNREACHABLE;
@@ -431,30 +429,16 @@ static wasm_val_t FromWabtValue(Store& store, const TypedValue& tv) {
       out_value.kind = WASM_F64;
       out_value.of.f64 = tv.value.Get<f64>();
       break;
-    case Type::Anyref:
-    case Type::Funcref:
-    case Type::Hostref:
-    case Type::Nullref: {
+    case Type::FuncRef: {
       Ref ref = tv.value.Get<Ref>();
-      // Get the actual type for this reference; tv.type uses the function
-      // signature, which may be to general (e.g. anyref).
-      Type type = store.GetValueType(ref);
-      out_value.kind = FromWabtValueType(type);
-      switch (type) {
-        case Type::Funcref:
-          out_value.of.ref = new wasm_func_t(store.UnsafeGet<Func>(ref));
-          break;
-
-        case Type::Hostref:
-          out_value.of.ref = new wasm_foreign_t(store.UnsafeGet<Foreign>(ref));
-          break;
-
-        case Type::Nullref:
-          out_value.of.ref = nullptr;
-          break;
-
-        default: WABT_UNREACHABLE;
-      }
+      out_value.kind = WASM_FUNCREF;
+      out_value.of.ref = new wasm_func_t(store.UnsafeGet<Func>(ref));
+      break;
+    }
+    case Type::ExternRef: {
+      Ref ref = tv.value.Get<Ref>();
+      out_value.kind = WASM_ANYREF;
+      out_value.of.ref = new wasm_foreign_t(store.UnsafeGet<Foreign>(ref));
       break;
     }
     default:
@@ -596,6 +580,12 @@ void wasm_trap_trace(const wasm_trap_t* trap, own wasm_frame_vec_t* out) {
   // TRACE(stderr, "error: %s\n", msg.c_str());
 }
 
+// wasm_config
+
+own wasm_config_t* wasm_config_new() {
+  return new wasm_config_t();
+}
+
 // wasm_engine
 
 own wasm_engine_t* wasm_engine_new() {
@@ -629,6 +619,28 @@ own wasm_module_t* wasm_module_new(wasm_store_t* store,
   return new wasm_module_t{Module::New(store->I, module_desc), binary};
 }
 
+bool wasm_module_validate(wasm_store_t* store, const wasm_byte_vec_t* binary) {
+  // TODO: Optimize this; for now it is generating a new module and discarding
+  // it. But since this call only needs to validate, it could do much less.
+  wasm_module_t* module = wasm_module_new(store, binary);
+  if (module == nullptr) {
+    return false;
+  }
+  wasm_module_delete(module);
+  return true;
+}
+
+void wasm_module_imports(const wasm_module_t* module,
+                         own wasm_importtype_vec_t* out) {
+  auto&& import_types = module->As<Module>()->import_types();
+  TRACE("%" PRIzx, import_types.size());
+  wasm_importtype_vec_new_uninitialized(out, import_types.size());
+
+  for (size_t i = 0; i < import_types.size(); ++i) {
+    out->data[i] = new wasm_importtype_t{import_types[i]};
+  }
+}
+
 void wasm_module_exports(const wasm_module_t* module,
                          own wasm_exporttype_vec_t* out) {
   auto&& export_types = module->As<Module>()->export_types();
@@ -943,6 +955,10 @@ own wasm_memory_t* wasm_memory_new(wasm_store_t* store,
   return new wasm_memory_t{Memory::New(store->I, *type->As<MemoryType>())};
 }
 
+own wasm_memorytype_t* wasm_memory_type(const wasm_memory_t* memory) {
+  return new wasm_memorytype_t{memory->As<Memory>()->type()};
+}
+
 byte_t* wasm_memory_data(wasm_memory_t* memory) {
   return reinterpret_cast<byte_t*>(memory->As<Memory>()->UnsafeData());
 }
