@@ -22,7 +22,7 @@
 
 typedef enum WabtOpcodeEnum {
 #define WABT_OPCODE(rtype, type1, type2, type3, mem_size, prefix, code, Name, \
-                    text)                                                     \
+                    text, decomp)                                             \
   Name,
 #include "opcode.def"
 #undef WABT_OPCODE
@@ -34,7 +34,7 @@ WABT_STATIC_ASSERT(Invalid <= WABT_OPCODE_CODE_TABLE_SIZE);
 /* The array index calculated below must match the one in Opcode::FromCode. */
 uint32_t WabtOpcodeCodeTable[WABT_OPCODE_CODE_TABLE_SIZE] = {
 #define WABT_OPCODE(rtype, type1, type2, type3, mem_size, prefix, code, Name, \
-                    text)                                                     \
+                    text, decomp)                                             \
   [(prefix << 8) + code] = Name,
 #include "opcode.def"
 #undef WABT_OPCODE
