@@ -39,7 +39,7 @@ static std::string s_infile;
 static std::string s_outfile;
 static Features s_features;
 static WriteWatOptions s_write_wat_options;
-static bool s_generate_names;
+static bool s_generate_names = false;
 static bool s_read_debug_names = true;
 static bool s_fail_on_custom_section_error = true;
 static std::unique_ptr<FileStream> s_log_stream;
