@@ -134,11 +134,18 @@ int ProgramMain(int argc, char** argv) {
                                    output_basename, s_write_binary_options,
                                    &module_streams, s_log_stream.get());
 
-    json_stream.WriteToFile(s_outfile);
+    if (Succeeded(result)) {
+      result = json_stream.WriteToFile(s_outfile);
+    }
 
-    for (auto iter = module_streams.begin(); iter != module_streams.end();
-         ++iter) {
-      iter->stream->WriteToFile(iter->filename);
+    if (Succeeded(result)) {
+      for (auto iter = module_streams.begin(); iter != module_streams.end();
+           ++iter) {
+        result = iter->stream->WriteToFile(iter->filename);
+        if (!Succeeded(result)) {
+          break;
+        }
+      }
     }
   }
 
