@@ -123,8 +123,10 @@ def main(args):
     parser.add_argument('--enable-bulk-memory', action='store_true')
     parser.add_argument('--enable-tail-call', action='store_true')
     parser.add_argument('--enable-reference-types', action='store_true')
+    parser.add_argument('--enable-memory64', action='store_true')
     parser.add_argument('--inline-exports', action='store_true')
     parser.add_argument('--inline-imports', action='store_true')
+    parser.add_argument('--reloc', action='store_true')
     parser.add_argument('file', help='test file.')
     options = parser.parse_args(args)
 
@@ -143,6 +145,8 @@ def main(args):
         '--enable-bulk-memory': options.enable_bulk_memory,
         '--enable-tail-call': options.enable_tail_call,
         '--enable-reference-types': options.enable_reference_types,
+        '--enable-memory64': options.enable_memory64,
+        '--reloc': options.reloc,
         '--no-check': options.no_check,
     })
 
@@ -161,6 +165,7 @@ def main(args):
         '--enable-tail-call': options.enable_tail_call,
         '--enable-reference-types': options.enable_reference_types,
         '--enable-threads': options.enable_threads,
+        '--enable-memory64': options.enable_memory64,
         '--inline-exports': options.inline_exports,
         '--inline-imports': options.inline_imports,
         '--no-debug-names': not options.debug_names,
